@@ -125,8 +125,22 @@
 #define FIELD_BS(name,dxf) FIELDG(name, BS, dxf)
 #define FIELD_BL(name,dxf) FIELDG(name, BL, dxf)
 #define FIELD_BLL(name,dxf) FIELDG(name, BLL, dxf)
-#define FIELD_BD(name,dxf) FIELDG(name, BD, dxf)
+#define FIELD_BD(name,dxf) \
+   FIELDG(name, BD, dxf); \
+   if (bit_isnan(_obj->name)) { \
+     LOG_ERROR("Invalid BD " #name); \
+     return DWG_ERR_VALUEOUTOFBOUNDS; \
+   }
+#define FIELD_BLh(name,dxf) \
+  { _obj->name = bit_read_BL(dat); \
+    LOG_TRACE(#name ": 0x%x [BL %d]\n", (unsigned)_obj->name, dxf); }
 #define FIELD_RC(name,dxf) FIELDG(name, RC, dxf)
+#define FIELD_RCu(name,dxf) \
+  { _obj->name = bit_read_RC(dat); \
+    LOG_TRACE(#name ": %u [RC %d]\n", (unsigned)((unsigned char)_obj->name), dxf); }
+#define FIELD_RCd(name,dxf) \
+  { _obj->name = bit_read_RC(dat); \
+    LOG_TRACE(#name ": %d [RC %d]\n", (int)((signed char)_obj->name), dxf); }
 #define FIELD_RS(name,dxf) FIELDG(name, RS, dxf)
 #define FIELD_RD(name,dxf) FIELDG(name, RD, dxf)
 #define FIELD_RL(name,dxf) FIELDG(name, RL, dxf)
@@ -270,12 +284,18 @@
     *dat = here;                                                \
   }
 
+#define VECTOR_CHKCOUNT(name,size) \
+  if (dat->version >= R_2004 && size > 0xff00) { \
+    LOG_ERROR("Invalid " #name " vcount %ld", (long)size); \
+    return DWG_ERR_VALUEOUTOFBOUNDS; } \
+
 //FIELD_VECTOR_N(name, type, size):
 // reads data of the type indicated by 'type' 'size' times and stores
 // it all in the vector called 'name'.
 #define FIELD_VECTOR_N(name, type, size, dxf) \
   if (size > 0) \
     { \
+      VECTOR_CHKCOUNT(name,size) \
       _obj->name = (BITCODE_##type*) malloc(size * sizeof(BITCODE_##type));\
       for (vcount=0; vcount<(long)size; vcount++) \
         {\
@@ -287,6 +307,7 @@
 #define FIELD_VECTOR_T(name, size, dxf) \
   if (_obj->size > 0) \
     { \
+      VECTOR_CHKCOUNT(name,_obj->size) \
       _obj->name = (char**) malloc(_obj->size * sizeof(char*)); \
       for (vcount=0; vcount<(long)_obj->size; vcount++) \
         {\
@@ -304,21 +325,27 @@
 #define FIELD_VECTOR(name, type, size, dxf) FIELD_VECTOR_N(name, type, _obj->size, dxf)
 
 #define FIELD_2RD_VECTOR(name, size, dxf)                                   \
+  VECTOR_CHKCOUNT(name,_obj->size) \
   _obj->name = (BITCODE_2RD *) malloc(_obj->size * sizeof(BITCODE_2RD));\
   for (vcount=0; vcount< (long)_obj->size; vcount++)\
     {\
       FIELD_2RD(name[vcount], dxf); \
     }
 
-#define FIELD_2DD_VECTOR(name, size, dxf)                                   \
+#define FIELD_2DD_VECTOR(name, size, dxf) \
+  VECTOR_CHKCOUNT(name,_obj->size) \
   _obj->name = (BITCODE_2RD *) malloc(_obj->size * sizeof(BITCODE_2RD));\
-  FIELD_2RD(name[0], dxf);                                                  \
+  FIELD_2RD(name[0], dxf); \
   for (vcount = 1; vcount < (long)_obj->size; vcount++)\
     {\
-      FIELD_2DD(name[vcount], FIELD_VALUE(name[vcount - 1].x), FIELD_VALUE(name[vcount - 1].y), dxf); \
+      FIELD_DD(name[vcount].x, FIELD_VALUE(name[vcount - 1].x), dxf); \
+      FIELD_DD(name[vcount].y, FIELD_VALUE(name[vcount - 1].x), dxf+10);\
+      LOG_TRACE(#name "[%ld]: (" FORMAT_BD ", " FORMAT_BD ") [DD %d]\n", \
+                (long)vcount, _obj->name[vcount].x, _obj->name[vcount].y, dxf) \
     }
 
-#define FIELD_3DPOINT_VECTOR(name, size, dxf)                               \
+#define FIELD_3DPOINT_VECTOR(name, size, dxf) \
+  VECTOR_CHKCOUNT(name,_obj->size) \
   _obj->name = (BITCODE_3DPOINT *) malloc(_obj->size * sizeof(BITCODE_3DPOINT));\
   for (vcount=0; vcount < (long)_obj->size; vcount++) \
     {\
@@ -326,6 +353,7 @@
     }
 
 #define HANDLE_VECTOR_N(name, size, code, dxf) \
+  VECTOR_CHKCOUNT(name,size) \
   FIELD_VALUE(name) = (BITCODE_H*) malloc(sizeof(BITCODE_H) * size);\
   for (vcount=0; vcount < (long)size; vcount++) \
     {\
@@ -413,22 +441,26 @@
     bit_set_position(hdl_dat, obj->hdlpos); \
     LOG_HANDLE(" -> @%lu.%u (%lu)\n", dat->byte, dat->bit, bit_position(dat)); }
 
+#define REPEAT_CHKCOUNT(name,times) \
+  if (dat->version >= R_2004 && times > 0xff00) { \
+    LOG_ERROR("Invalid " #name " rcount %ld\n", (long)times); \
+    return DWG_ERR_VALUEOUTOFBOUNDS; } \
+
 // unchecked with a constant
 #define REPEAT_CN(times, name, type) \
   _obj->name = (type *) calloc(times, sizeof(type)); \
   for (rcount1=0; rcount1<(long)times; rcount1++)
 #define REPEAT_N(times, name, type) \
-  if (dat->version >= R_2010 && times > 0x1000) { \
-    fprintf(stderr, "Invalid rcount %ld", (long)times); return DWG_ERR_VALUEOUTOFBOUNDS; } \
+  REPEAT_CHKCOUNT(name,times) \
   if (times) _obj->name = (type *) calloc(times, sizeof(type)); \
   for (rcount1=0; rcount1<(long)times; rcount1++)
 
 #define _REPEAT(times, name, type, idx) \
-  if (dat->version >= R_2010 && _obj->times > 0x1000) { \
-    fprintf(stderr, "Invalid rcount " #idx " %ld", (long)_obj->times); return DWG_ERR_VALUEOUTOFBOUNDS; } \
+  REPEAT_CHKCOUNT(name,_obj->times) \
   if (_obj->times) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
   for (rcount##idx=0; rcount##idx<(long)_obj->times; rcount##idx++)
 #define _REPEAT_C(times, name, type, idx) \
+  REPEAT_CHKCOUNT(name,_obj->times) \
   if (_obj->times) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
   for (rcount##idx=0; rcount##idx<(long)_obj->times; rcount##idx++)
 
